package Acrux;
use warnings;
use strict;
use utf8;

=encoding utf-8

=head1 NAME

Acrux - Southern crucis constellation for your applications

=head1 SYNOPSIS

    use Acrux;

=head1 DESCRIPTION

Southern crucis constellation for your applications

See L<Acme::Crux> for more!

=head1 TO DO

See C<TODO> file

=head1 SEE ALSO

L<Acme::Crux>, L<CTK>

=head1 AUTHOR

Serż Minus (Sergey Lepenkov) L<https://www.serzik.com> E<lt>abalama@cpan.orgE<gt>

=head1 COPYRIGHT

Copyright (C) 1998-2026 D&D Corporation

=head1 LICENSE

This program is distributed under the terms of the Artistic License Version 2.0

See the C<LICENSE> file or L<https://opensource.org/license/artistic-2-0> for details

=cut

our $VERSION = '0.10';

1;

__END__
